function [thetahat,xhat]=nlms(x,y,N,muu)

% [thetahat,xhat]=nlms(x,y,N,muu)
%
%	x			- Data sequence
%	y			- Data sequence
%	N			- Model order 
%	muu			- Step size
%	thetahat		- Matrix with estimates of theta. 
%				  Row n corresponds to the estimate thetahat(n)'
%	xhat			- Estimate of x
%
%  nlms: The Normalized Least-Mean Square Algorithm
%
% 	Estimator: xhat(n)=Y^{T}(n)thetahat(n-1)
%
%	thetahat is estimated using NLMS. 
%     
%     Author: Manuel Forrer, Markus Frhle
%     Date: 2009-02-22
%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Initialize xhat and thetahat
% Loop
M = size(y,1);
xhat = zeros(M,1);
thetahat = zeros(M,N);
Y = zeros(N,1);
X = zeros(N,1);
Xhat = zeros(N,1);


for n=1:M,

	% Generate Y. Set elements of Y that does not exist to zero
    Y = [y(n); Y(1:end-1)];
	% Estimate of x

    xhat(n,1) = Y' * thetahat(n,:)';


	% Update the n+1 row in the matrix thetahat which in the notation in the Lecture Notes
	% corresponds to thetahat(n)

	thetahat(n+1,:)= thetahat(n,:) + (muu * (1+norm(Y)^2)^(-1) .* Y *( x(n) - xhat(n,1) ))';
end

% Shift thetahat one step so that row n corresponds to time n

thetahat=thetahat(2:M+1,:);
