% Advanced Signal Processing Seminar
% Manuel forrer, Markus Froehle
%
% Use LMS to identify an 
% unknown 32nd-order FIR filter.

clear all;
close all;

% Initialization
x  = randn(1,500);      % Input to the filter 
n  = 0.1*randn(1,500);  % Observation noise signal 

N = 31;                 % filter order
cutoff = 0.5;           % cut-off frequency between 0 and 1
mu = 0.008;             % LMS step size. 

% FIR system to be identified 
b  = fir1(N, cutoff);

% Desired signal   
d  = filter(b, 1, x)+n;  

% Apply LMS to calculate filter coefficients, output and error
lms = adaptfilt.lms(N+1, mu); 
[y_lms,e_lms] = filter(lms, x, d); 

% Output
figure
subplot(3,1,1); plot(1:500,[d;y_lms]); 
title('System Identification of an FIR Filter with LMS'); 
legend('Desired','Output'); 
xlabel('Time Index'); ylabel('Signal Value'); 
%subplot(3,1,2); hist(abs(e),500);
subplot(3,1,2); plot(abs(e_lms));
legend('|Error_{LMS}|');
xlabel('Time Index'); ylabel('|Error|');
ylim([0 1.6]);
subplot(3,1,3); stem([b.', lms.coefficients.']); 
legend('Actual','Estimated'); 
xlabel('Coefficient #'); ylabel('Coefficient Value');  grid on;