% Advanced Signal Processing Seminar
% Manuel forrer, Markus Froehle
%
% Use 500 iterations of an adapting filter system (RLS) to identify and 
% unknown 32nd-order FIR filter.
clear all;
close all;

% Initialization
x  = randn(1,500);      % Input to the filter
n  = 0.1*randn(1,500);  % Observation noise signal 

N = 31;                 % filter order
cutoff = 0.5;           % cut-off frequency between 0 and 1
P0 = 10*eye(32);        % Initial sqrt correlation matrix inverse 
lamda = 0.99;           % RLS forgetting factor 

% FIR system to be identified 
b  = fir1(N, cutoff);  

% Desired signal  
d  = filter(b, 1, x)+n;

% Applz RLS
rls = adaptfilt.rls(N+1, lamda, P0); 
[y_rls, e_rls] = filter(rls, x, d); 

%Output
subplot(3,1,1); plot(1:500,[d;y_rls]);
title('System Identification of an FIR Filter with RLS'); 
legend('Desired','Output'); 
xlabel('Time Index'); ylabel('Signal Value'); 
%subplot(3,1,2); hist(abs(e),500);
subplot(3,1,2); plot(abs(e_rls));
legend('|Error_{RLS}|'); 
xlabel('Time Index'); ylabel('|Error|'); 
subplot(3,1,3); stem([b.', rls.Coefficients.']); 
legend('Actual','Estimated');  
xlabel('Coefficient #'); ylabel('Coefficient Value');  grid on;